//! This module creates the defaultable version of a structured derived with
//! Georm. It creates a new struct named `<StructName>Default` where the fields
//! marked as defaultable become an `Option<T>`, where `T` is the initial type
//! of the field.
//!
//! The user does not have to mark a field defaultable if the field already has
//! a type `Option<T>`. It is intended only for fields marked as `NOT NULL` in
//! the database, but not required when creating the entity due to a `DEFAULT`
//! or something similar. The type `<StructName>Default` implements the
//! `Defaultable` trait.

use super::ir::{GeormField, GeormStructAttributes};
use quote::quote;

fn create_defaultable_field(field: &GeormField) -> proc_macro2::TokenStream {
    let ident = &field.ident;
    let ty = &field.ty;
    let vis = &field.field.vis;

    // If the field is marked as defaultable, wrap it in Option<T>
    // Otherwise, keep the original type
    let field_type = if field.defaultable {
        quote! { Option<#ty> }
    } else {
        quote! { #ty }
    };

    quote! {
        #vis #ident: #field_type
    }
}

fn generate_defaultable_trait_impl(
    struct_name: &syn::Ident,
    defaultable_struct_name: &syn::Ident,
    struct_attrs: &GeormStructAttributes,
    fields: &[GeormField],
) -> proc_macro2::TokenStream {
    let table = &struct_attrs.table;

    // Find the ID field
    let id_field = fields
        .iter()
        .find(|field| field.id)
        .expect("Must have an ID field");
    let id_type = &id_field.ty;

    // Generate the INSERT query for defaultable struct
    let field_names: Vec<String> = fields.iter().map(|f| f.ident.to_string()).collect();
    let field_placeholders: Vec<String> =
        (1..=field_names.len()).map(|i| format!("${}", i)).collect();

    let insert_query = format!(
        "INSERT INTO {} ({}) VALUES ({}) RETURNING *",
        table,
        field_names.join(", "),
        field_placeholders.join(", ")
    );

    // Generate field identifiers for binding
    let field_idents: Vec<&syn::Ident> = fields.iter().map(|field| &field.ident).collect();

    quote! {
        impl ::georm::Defaultable<#id_type, #struct_name> for #defaultable_struct_name {
            async fn create(&self, pool: &::sqlx::PgPool) -> ::sqlx::Result<#struct_name> {
                ::sqlx::query_as!(
                    #struct_name,
                    #insert_query,
                    #(self.#field_idents),*
                )
                .fetch_one(pool)
                .await
            }
        }
    }
}

pub fn derive_defaultable_struct(
    ast: &syn::DeriveInput,
    struct_attrs: &GeormStructAttributes,
    fields: &[GeormField],
) -> proc_macro2::TokenStream {
    // Only generate if there are defaultable fields
    if fields.iter().all(|field| !field.defaultable) {
        return quote! {};
    }

    let struct_name = &ast.ident;
    let vis = &ast.vis;
    let defaultable_struct_name = quote::format_ident!("{}Default", struct_name);

    let defaultable_fields: Vec<proc_macro2::TokenStream> =
        fields.iter().map(create_defaultable_field).collect();

    let trait_impl = generate_defaultable_trait_impl(
        struct_name,
        &defaultable_struct_name,
        struct_attrs,
        fields,
    );

    quote! {
        #[derive(Debug, Clone)]
        #vis struct #defaultable_struct_name {
            #(#defaultable_fields),*
        }

        #trait_impl
    }
}
